@@ -3,7 +3,7 @@ Simple text input
 """
 from __future__ import annotations
 
-from typing import TYPE_CHECKING, Any, List, Optional, Tuple, Union
+from typing import TYPE_CHECKING, List, Optional, Tuple, Union
 
 import rich.box
 from rich.panel import Panel
@@ -13,11 +13,29 @@ from textual import events
 from textual.reactive import Reactive
 from textual.widget import Widget
 
-from textual_inputs.events import InputOnChange, InputOnFocus
+from textual_inputs.events import InputOnChange, InputOnFocus, make_message_class
+
+from rich.console import Console
+from rich.syntax import Syntax
 
 if TYPE_CHECKING:
     from rich.console import RenderableType
 
+CONSOLE = Console()
+
+
+def conceal_text(segment: str) -> str:
+    """Produce the segment concealed like a password."""
+    return "•" * len(segment)
+
+
+def syntax_highlight_text(code: str, syntax: str) -> Text:
+    """Produces highlighted text based on the syntax."""
+    syntax_obj = Syntax(code, syntax)
+    with CONSOLE.capture() as capture:
+        CONSOLE.print(syntax_obj)
+    return Text.from_ansi(capture.get())
+
 
 class TextInput(Widget):
     """
@@ -33,17 +51,25 @@ class TextInput(Widget):
             of the widget's border.
         password (bool, optional): Defaults to False. Hides the text
             input, replacing it with bullets.
+        syntax (Optional[str]): the name of the language for syntax highlighting.
 
     Attributes:
         value (str): the value of the text field
         placeholder (str): The placeholder message.
         title (str): The displayed title of the widget.
         has_password (bool): True if the text field masks the input.
+        syntax (Optional[str]): the name of the language for syntax highlighting.
         has_focus (bool): True if the widget is focused.
         cursor (Tuple[str, Style]): The character used for the cursor
             and a rich Style object defining its appearance.
+        on_change_handler_name (str): name of handler function to be
+            called when an on change event occurs. Defaults to
+            handle_input_on_change.
+        on_focus_handler_name (name): name of handler function to be
+            called when an on focus event occurs. Defaults to
+            handle_input_on_focus.
 
-    Messages:
+    Events:
         InputOnChange: Emitted when the contents of the input changes.
         InputOnFocus: Emitted when the widget becomes focused.
 
@@ -81,13 +107,17 @@ class TextInput(Widget):
         placeholder: str = "",
         title: str = "",
         password: bool = False,
+        syntax: Optional[str] = None,
         **kwargs: Any,
     ) -> None:
-        super().__init__(name, **kwargs)
+        super().__init__(name)
         self.value = value
         self.placeholder = placeholder
         self.title = title
         self.has_password = password
+        self.syntax = syntax
+        self._on_change_message_class = InputOnChange
+        self._on_focus_message_class = InputOnFocus
         self._cursor_position = len(self.value)
 
     def __rich_repr__(self):
@@ -98,12 +128,40 @@ class TextInput(Widget):
         else:
             value = self.value
         yield "value", value
+        yield "on_change_handler_name", self.on_change_handler_name
+        yield "on_focus_handler_name", self.on_focus_handler_name
 
     @property
     def has_focus(self) -> bool:
         """Produces True if widget is focused"""
         return self._has_focus
 
+    @property
+    def on_change_handler_name(self) -> str:
+        return self._on_change_message_class._handler
+
+    @on_change_handler_name.setter
+    def on_change_handler_name(self, handler_name: str) -> None:
+        """
+        Name of the handler function for an on change event to be sent to.
+        Must start with "handle_" and contain only lowercase ASCII letters,
+        numbers and underscores.
+        """
+        self._on_change_message_class = make_message_class(handler_name)
+
+    @property
+    def on_focus_handler_name(self) -> str:
+        return self._on_focus_message_class._handler
+
+    @on_focus_handler_name.setter
+    def on_focus_handler_name(self, handler_name: str) -> None:
+        """
+        Name of the handler function for an on focus event to be sent to.
+        Must start with "handle_" and contain only lowercase ASCII letters,
+        numbers and underscores.
+        """
+        self._on_focus_message_class = make_message_class(handler_name)
+
     def render(self) -> RenderableType:
         """
         Produce a Panel object containing placeholder text or value
@@ -118,7 +176,7 @@ class TextInput(Widget):
                 else:
                     segments = [self.placeholder]
             else:
-                segments = [self._conceal_or_reveal(self.value)]
+                segments = [self._modify_text(self.value)]
 
         text = Text.assemble(*segments)
 
@@ -142,33 +200,26 @@ class TextInput(Widget):
             box=rich.box.DOUBLE if self.has_focus else rich.box.SQUARE,
         )
 
-    def _conceal_or_reveal(self, segment: str) -> str:
+    def _modify_text(self, segment: str) -> Union[str, Text]:
         """
-        Produce the segment either concealed like a password or as it
-        was passed.
+        Produces the text with modifications, such as password concealing.
         """
         if self.has_password:
-            return "".join("•" for _ in segment)
+            return conceal_text(segment)
+        if self.syntax:
+            return syntax_highlight_text(segment, self.syntax)
         return segment
 
-    def _render_text_with_cursor(self) -> List[Union[str, Tuple[str, Style]]]:
+    def _render_text_with_cursor(self) -> List[Union[str, Text, Tuple[str, Style]]]:
         """
         Produces the renderable Text object combining value and cursor
         """
-        if len(self.value) == 0:
-            segments = [self.cursor]
-        elif self._cursor_position == 0:
-            segments = [self.cursor, self._conceal_or_reveal(self.value)]
-        elif self._cursor_position == len(self.value):
-            segments = [self._conceal_or_reveal(self.value), self.cursor]
-        else:
-            segments = [
-                self._conceal_or_reveal(self.value[: self._cursor_position]),
-                self.cursor,
-                self._conceal_or_reveal(self.value[self._cursor_position :]),
-            ]
-
-        return segments
+        text = self._modify_text(self.value)
+        return [
+            text[: self._cursor_position],
+            self.cursor,
+            text[self._cursor_position :],
+        ]
 
     async def on_focus(self, event: events.Focus) -> None:
         self._has_focus = True
@@ -262,7 +313,7 @@ class TextInput(Widget):
 
     async def _emit_on_change(self, event: events.Key) -> None:
         event.stop()
-        await self.emit(InputOnChange(self))
+        await self.emit(self._on_change_message_class(self))
 
     async def _emit_on_focus(self) -> None:
-        await self.emit(InputOnFocus(self))
+        await self.emit(self._on_focus_message_class(self))
